@@ -37,6 +37,7 @@ from argparse import ArgumentParser
 
 import importlib
 from onnx2tf.utils.colors import Color
+from sng4onnx import generate as op_name_auto_generate
 
 
 def convert(
@@ -44,12 +45,17 @@ def convert(
     onnx_graph: Optional[onnx.ModelProto] = None,
     output_folder_path: Optional[str] = 'saved_model',
     output_signaturedefs: Optional[bool] = False,
+    output_h5: Optional[bool] = False,
     not_use_onnxsim: Optional[bool] = False,
+    not_use_opname_auto_generate: Optional[bool] = False,
     batch_size: Optional[int] = None,
     overwrite_input_shape: Optional[List[str]] = None,
     keep_ncw_or_nchw_or_ncdhw_input_names: Optional[List[str]] = None,
     replace_argmax_to_reducemax_and_indicies_is_int64: Optional[bool] = False,
     replace_argmax_to_reducemax_and_indicies_is_float32: Optional[bool] = False,
+    replace_argmax_to_fused_argmax_and_indicies_is_int64: Optional[bool] = False,
+    replace_argmax_to_fused_argmax_and_indicies_is_float32: Optional[bool] = False,
+    fused_argmax_scale_ratio: Optional[float] = 0.5,
     replace_asin_to_pseudo_asin: Optional[bool] = False,
     replace_acos_to_pseudo_acos: Optional[bool] = False,
     replace_leakyrelu_to_pseudo_leakyrelu: Optional[bool] = False,
@@ -83,10 +89,17 @@ def convert(
         to other model formats. However, this can significantly reduce the speed\n
         of model conversion and significant increase the size of the model.
 
+    output_h5: Optional[bool]
+        Output in Keras H5 format.
+
     not_use_onnxsim: Optional[bool]
         No optimization by onnx-simplifier is performed.\n
         If this option is used, the probability of a conversion error is very high.
 
+    not_use_opname_auto_generate: Optional[bool]
+        Automatic generation of each OP name in the old format ONNX file\n
+        and assignment of OP name are not performed.
+
     batch_size: Optional[int]
         Fixes the dynamic batch size to the specified numeric batch size.\n
         A value of 1 or more must be specified.
@@ -113,15 +126,47 @@ def convert(
     replace_argmax_to_reducemax_and_indicies_is_int64: Optional[bool]
         Replace ArgMax with a ReduceMax. The returned indicies are int64.\n
         Only one of replace_argmax_to_reducemax_and_indicies_is_int64 and \n
-        replace_argmax_to_reducemax_and_indicies_is_float32 can be specified.\n
+        replace_argmax_to_reducemax_and_indicies_is_float32 and \n
+        replace_argmax_to_fused_argmax_and_indicies_is_int64 and \n
+        replace_argmax_to_fused_argmax_and_indicies_is_float32 can be specified.\n
         Default: False
 
     replace_argmax_to_reducemax_and_indicies_is_float32: Optional[bool]
         Replace ArgMax with a ReduceMax. The returned indicies are float32.\n
         Only one of replace_argmax_to_reducemax_and_indicies_is_int64 and \n
-        replace_argmax_to_reducemax_and_indicies_is_float32 can be specified.\n
+        replace_argmax_to_reducemax_and_indicies_is_float32 and \n
+        replace_argmax_to_fused_argmax_and_indicies_is_int64 and \n
+        replace_argmax_to_fused_argmax_and_indicies_is_float32 can be specified.\n
         Default: False
 
+    replace_argmax_to_fused_argmax_and_indicies_is_int64: Optional[bool]
+        Replace ArgMax with a ReduceMax. The returned indicies are int64.\n
+        It improves inference speed at the cost of a small sacrifice in accuracy.\n
+        See. https://github.com/tensorflow/models/tree/master/official/projects/edgetpu/vision#argmax-fusion-to-improve-segmentation-model-latency\n
+        Currently, only 4D tensors are supported.\n
+        Only one of replace_argmax_to_reducemax_and_indicies_is_int64 and \n
+        replace_argmax_to_reducemax_and_indicies_is_float32 and \n
+        replace_argmax_to_fused_argmax_and_indicies_is_int64 and \n
+        replace_argmax_to_fused_argmax_and_indicies_is_float32 can be specified.\n
+        Default: False
+
+    replace_argmax_to_fused_argmax_and_indicies_is_float32: Optional[bool]
+        Replace ArgMax with a ReduceMax. The returned indicies are float32.\n
+        It improves inference speed at the cost of a small sacrifice in accuracy.\n
+        See. https://github.com/tensorflow/models/tree/master/official/projects/edgetpu/vision#argmax-fusion-to-improve-segmentation-model-latency\n
+        Currently, only 4D tensors are supported.\n
+        Only one of replace_argmax_to_reducemax_and_indicies_is_int64 and \n
+        replace_argmax_to_reducemax_and_indicies_is_float32 and \n
+        replace_argmax_to_fused_argmax_and_indicies_is_int64 and \n
+        replace_argmax_to_fused_argmax_and_indicies_is_float32 can be specified.\n
+        Default: False
+
+    fused_argmax_scale_ratio: Optional[float]
+        For Fused ArgMax.\n
+        Scale ratio when generating Fused ArgMax.\n
+        0.0 < fused_argmax_scale_ratio < 1.0\n
+        Default: 0.5
+
     replace_asin_to_pseudo_asin: Optional[bool]
         Replace Asin with a pseudo Asin.
 
@@ -208,12 +253,30 @@ def convert(
 
     # replace_argmax_to_reducemax_and_indicies_is_int64
     # replace_argmax_to_reducemax_and_indicies_is_float32
-    if replace_argmax_to_reducemax_and_indicies_is_int64 \
-        and replace_argmax_to_reducemax_and_indicies_is_float32:
+    # replace_argmax_to_fused_argmax_and_indicies_is_int64
+    # replace_argmax_to_fused_argmax_and_indicies_is_float32
+    ra_option_list = [
+        replace_argmax_to_reducemax_and_indicies_is_int64,
+        replace_argmax_to_reducemax_and_indicies_is_float32,
+        replace_argmax_to_fused_argmax_and_indicies_is_int64,
+        replace_argmax_to_fused_argmax_and_indicies_is_float32,
+    ]
+    if ra_option_list.count(True) > 1:
         print(
             f'{Color.RED}ERROR:{Color.RESET} ' +
             f'Only one of replace_argmax_to_reducemax_and_indicies_is_int64 and ' +
-            f'replace_argmax_to_reducemax_and_indicies_is_float32 can be specified.'
+            f'replace_argmax_to_reducemax_and_indicies_is_float32 and ' +
+            f'replace_argmax_to_fused_argmax_and_indicies_is_int64 and ' +
+            f'replace_argmax_to_fused_argmax_and_indicies_is_float32 can be specified.'
+        )
+        sys.exit(1)
+
+    # fused_argmax_scale_ratio
+    if ra_option_list.count(True) > 0 and not (0.0 < fused_argmax_scale_ratio < 1.0):
+        print(
+            f'fused_argmax_scale_ratio must be specified in the range '+
+            f'0.0 < fused_argmax_scale_ratio < 1.0. '+
+            f'fused_argmax_scale_ratio: {fused_argmax_scale_ratio}'
         )
         sys.exit(1)
 
@@ -270,6 +333,19 @@ def convert(
                 import traceback
                 traceback.print_exc()
 
+    # Automatic generation of each OP name - sng4onnx
+    if not not_use_opname_auto_generate:
+        if not non_verbose:
+            print('')
+            print(f'{Color.REVERCE}Automatic generation of each OP name started{Color.RESET}', '=' * 40)
+        op_name_auto_generate(
+            input_onnx_file_path=f'{input_onnx_file_path}',
+            output_onnx_file_path=f'{input_onnx_file_path}',
+            non_verbose=True,
+        )
+        if not non_verbose:
+            print(f'{Color.GREEN}Automatic generation of each OP name complete!{Color.RESET}')
+
     # Loading Graphs
     # onnx_graph If specified, onnx_graph is processed first
     if not onnx_graph:
@@ -291,6 +367,9 @@ def convert(
         'non_verbose': non_verbose,
         'replace_argmax_to_reducemax_and_indicies_is_int64': replace_argmax_to_reducemax_and_indicies_is_int64,
         'replace_argmax_to_reducemax_and_indicies_is_float32': replace_argmax_to_reducemax_and_indicies_is_float32,
+        'replace_argmax_to_fused_argmax_and_indicies_is_int64': replace_argmax_to_fused_argmax_and_indicies_is_int64,
+        'replace_argmax_to_fused_argmax_and_indicies_is_float32': replace_argmax_to_fused_argmax_and_indicies_is_float32,
+        'fused_argmax_scale_ratio': fused_argmax_scale_ratio,
         'replace_asin_to_pseudo_asin': replace_asin_to_pseudo_asin,
         'replace_acos_to_pseudo_acos': replace_acos_to_pseudo_acos,
         'replace_leakyrelu_to_pseudo_leakyrelu': replace_leakyrelu_to_pseudo_leakyrelu,
@@ -373,6 +452,14 @@ def convert(
             model.summary(line_length=140)
             print('')
 
+        # Output in Keras H5 format
+        if output_h5:
+            if not non_verbose:
+                print(f'{Color.REVERCE}h5 output started{Color.RESET}', '=' * 67)
+            model.save(f'{output_folder_path}/model_float32.h5')
+            if not non_verbose:
+                print(f'{Color.GREEN}h5 output complete!{Color.RESET}')
+
         # Create concrete func
         run_model = tf.function(lambda *inputs : model(inputs))
         concrete_func = run_model.get_concrete_function(
@@ -432,13 +519,19 @@ def convert(
 
 def main():
     parser = ArgumentParser()
-    parser.add_argument(
+    iV_group = parser.add_mutually_exclusive_group(required=True)
+    iV_group.add_argument(
         '-i',
         '--input_onnx_file_path',
         type=str,
-        required=True,
         help='Input onnx file path.'
     )
+    iV_group.add_argument(
+        '-V',
+        '--version',
+        action='store_true',
+        help='Show version and exit.'
+    )
     parser.add_argument(
         '-o',
         '--output_folder_path',
@@ -456,6 +549,13 @@ def main():
             'to other model formats. However, this can significantly reduce the speed \n' +
             'of model conversion and significant increase the size of the model.'
     )
+    parser.add_argument(
+        '-oh5',
+        '--output_h5',
+        action='store_true',
+        help=\
+            'Output in Keras H5 format.'
+    )
     parser.add_argument(
         '-nuo',
         '--not_use_onnxsim',
@@ -464,6 +564,14 @@ def main():
             'No optimization by onnx-simplifier is performed. \n' +
             'If this option is used, the probability of a conversion error is very high.'
     )
+    parser.add_argument(
+        '-nuonag',
+        '--not_use_opname_auto_generate',
+        action='store_true',
+        help=\
+            'Automatic generation of each OP name in the old format ONNX file '+
+            'and assignment of OP name are not performed.'
+    )
     parser.add_argument(
         '-b',
         '--batch_size',
@@ -509,7 +617,9 @@ def main():
         help=\
             'Replace ArgMax with a ReduceMax. The returned indicies are int64. \n' +
             'Only one of replace_argmax_to_reducemax_and_indicies_is_int64 and \n' +
-            'replace_argmax_to_reducemax_and_indicies_is_float32 can be specified.'
+            'replace_argmax_to_reducemax_and_indicies_is_float32 and \n'+
+            'replace_argmax_to_fused_argmax_and_indicies_is_int64 and \n'+
+            'replace_argmax_to_fused_argmax_and_indicies_is_float32 can be specified.'
     )
     rar_group.add_argument(
         '-rarf32',
@@ -518,7 +628,48 @@ def main():
         help=\
             'Replace ArgMax with a ReduceMax. The returned indicies are float32. \n' +
             'Only one of replace_argmax_to_reducemax_and_indicies_is_int64 and \n' +
-            'replace_argmax_to_reducemax_and_indicies_is_float32 can be specified.'
+            'replace_argmax_to_reducemax_and_indicies_is_float32 and \n'+
+            'replace_argmax_to_fused_argmax_and_indicies_is_int64 and \n'+
+            'replace_argmax_to_fused_argmax_and_indicies_is_float32 can be specified.'
+    )
+    rar_group.add_argument(
+        '-rafi64',
+        '--replace_argmax_to_fused_argmax_and_indicies_is_int64',
+        action='store_true',
+        help=\
+            'Replace ArgMax with a Fused_ArgMax. The returned indicies are int64. \n' +
+            'It improves inference speed at the cost of a small sacrifice in accuracy. \n' +
+            'See. https://github.com/tensorflow/models/tree/master/official/projects/edgetpu/vision#argmax-fusion-to-improve-segmentation-model-latency \n' +
+            'Currently, only 4D tensors are supported. \n' +
+            'Only one of replace_argmax_to_reducemax_and_indicies_is_int64 and \n' +
+            'replace_argmax_to_reducemax_and_indicies_is_float32 and \n'+
+            'replace_argmax_to_fused_argmax_and_indicies_is_int64 and \n'+
+            'replace_argmax_to_fused_argmax_and_indicies_is_float32 can be specified.'
+    )
+    rar_group.add_argument(
+        '-raff32',
+        '--replace_argmax_to_fused_argmax_and_indicies_is_float32',
+        action='store_true',
+        help=\
+            'Replace ArgMax with a Fused_ArgMax. The returned indicies are float32. \n' +
+            'It improves inference speed at the cost of a small sacrifice in accuracy. \n' +
+            'See. https://github.com/tensorflow/models/tree/master/official/projects/edgetpu/vision#argmax-fusion-to-improve-segmentation-model-latency \n' +
+            'Currently, only 4D tensors are supported. \n' +
+            'Only one of replace_argmax_to_reducemax_and_indicies_is_int64 and \n' +
+            'replace_argmax_to_reducemax_and_indicies_is_float32 and \n'+
+            'replace_argmax_to_fused_argmax_and_indicies_is_int64 and \n'+
+            'replace_argmax_to_fused_argmax_and_indicies_is_float32 can be specified.'
+    )
+    parser.add_argument(
+        '-fasr',
+        '--fused_argmax_scale_ratio',
+        type=float,
+        default=0.5,
+        help=\
+            'For Fused ArgMax. \n' +
+            'Scale ratio when generating Fused ArgMax. \n' +
+            '0.0 < fused_argmax_scale_ratio < 1.0 \n' +
+            'Default: 0.5'
     )
     parser.add_argument(
         '-rasin',
@@ -588,17 +739,27 @@ def main():
     )
     args = parser.parse_args()
 
+    # Print version
+    if args.version:
+        print(__version__)
+        sys.exit(0)
+
     # Convert
     model = convert(
         input_onnx_file_path=args.input_onnx_file_path,
         output_folder_path=args.output_folder_path,
         output_signaturedefs=args.output_signaturedefs,
+        output_h5=args.output_h5,
         not_use_onnxsim=args.not_use_onnxsim,
+        not_use_opname_auto_generate=args.not_use_opname_auto_generate,
         batch_size=args.batch_size,
         overwrite_input_shape=args.overwrite_input_shape,
         keep_ncw_or_nchw_or_ncdhw_input_names=args.keep_ncw_or_nchw_or_ncdhw_input_names,
         replace_argmax_to_reducemax_and_indicies_is_int64=args.replace_argmax_to_reducemax_and_indicies_is_int64,
         replace_argmax_to_reducemax_and_indicies_is_float32=args.replace_argmax_to_reducemax_and_indicies_is_float32,
+        replace_argmax_to_fused_argmax_and_indicies_is_int64=args.replace_argmax_to_fused_argmax_and_indicies_is_int64,
+        replace_argmax_to_fused_argmax_and_indicies_is_float32=args.replace_argmax_to_fused_argmax_and_indicies_is_float32,
+        fused_argmax_scale_ratio=args.fused_argmax_scale_ratio,
         replace_asin_to_pseudo_asin=args.replace_asin_to_pseudo_asin,
         replace_acos_to_pseudo_acos=args.replace_acos_to_pseudo_acos,
         replace_leakyrelu_to_pseudo_leakyrelu=args.replace_leakyrelu_to_pseudo_leakyrelu,
